package sun.security.rsa;

import java.util.Collections;
import java.util.HashMap;
import sun.security.jca.JCAUtil;
import javax.crypto.BadPaddingException;
import java.security.NoSuchAlgorithmException;
import javax.crypto.spec.PSource;
import java.security.spec.MGF1ParameterSpec;
import javax.crypto.spec.OAEPParameterSpec;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.util.Map;
import java.security.MessageDigest;
import java.security.SecureRandom;

public final class RSAPadding
{
    public static final int PAD_BLOCKTYPE_1 = 1;
    public static final int PAD_BLOCKTYPE_2 = 2;
    public static final int PAD_NONE = 3;
    public static final int PAD_OAEP_MGF1 = 4;
    private final int type;
    private final int paddedSize;
    private SecureRandom random;
    private final int maxDataSize;
    private MessageDigest md;
    private MGF1 mgf;
    private byte[] lHash;
    private static final Map<String, byte[]> emptyHashes;
    
    public static RSAPadding getInstance(int n, int n2) throws InvalidKeyException, InvalidAlgorithmParameterException {
        return new RSAPadding(n, n2, null, null);
    }
    
    public static RSAPadding getInstance(int n, int n2, SecureRandom secureRandom) throws InvalidKeyException, InvalidAlgorithmParameterException {
        return new RSAPadding(n, n2, secureRandom, null);
    }
    
    public static RSAPadding getInstance(int n, int n2, SecureRandom secureRandom, OAEPParameterSpec oaepParameterSpec) throws InvalidKeyException, InvalidAlgorithmParameterException {
        return new RSAPadding(n, n2, secureRandom, oaepParameterSpec);
    }
    
    private RSAPadding(int n, int n2, SecureRandom random, OAEPParameterSpec oaepParameterSpec) throws InvalidKeyException, InvalidAlgorithmParameterException {
        this.type = n;
        this.paddedSize = n2;
        this.random = random;
        if (n2 < 64) {
            throw new InvalidKeyException("Padded size must be at least 64");
        }
        switch (n) {
            case 1:
            case 2: {
                this.maxDataSize = n2 - 11;
                break;
            }
            case 3: {
                this.maxDataSize = n2;
                break;
            }
            case 4: {
                String s = "SHA-1";
                byte[] value = null;
                try {
                    if (oaepParameterSpec != null) {
                        s = oaepParameterSpec.getDigestAlgorithm();
                        String mgfAlgorithm = oaepParameterSpec.getMGFAlgorithm();
                        if (!mgfAlgorithm.equalsIgnoreCase("MGF1")) {
                            throw new InvalidAlgorithmParameterException("Unsupported MGF algo: " + mgfAlgorithm);
                        }
                        s = ((MGF1ParameterSpec)oaepParameterSpec.getMGFParameters()).getDigestAlgorithm();
                        PSource pSource = oaepParameterSpec.getPSource();
                        String algorithm = pSource.getAlgorithm();
                        if (!algorithm.equalsIgnoreCase("PSpecified")) {
                            throw new InvalidAlgorithmParameterException("Unsupported pSource algo: " + algorithm);
                        }
                        value = ((PSource.PSpecified)pSource).getValue();
                    }
                    this.md = MessageDigest.getInstance(s);
                    this.mgf = new MGF1(s);
                }
                catch (NoSuchAlgorithmException cause) {
                    throw new InvalidKeyException("Digest not available", cause);
                }
                this.lHash = getInitialHash(this.md, value);
                this.maxDataSize = n2 - 2 - 2 * this.lHash.length;
                if (this.maxDataSize <= 0) {
                    throw new InvalidKeyException("Key is too short for encryption using OAEPPadding with " + s + " and " + this.mgf.getName());
                }
                break;
            }
            default: {
                throw new InvalidKeyException("Invalid padding: " + n);
            }
        }
        return;
    }
    
    private static byte[] getInitialHash(MessageDigest messageDigest, byte[] input) {
        if (input == null || input.length == 0) {
            String algorithm = messageDigest.getAlgorithm();
            byte[] array = (byte[])RSAPadding.emptyHashes.get(algorithm);
            if (array == null) {
                array = messageDigest.digest();
                RSAPadding.emptyHashes.put(algorithm, array);
            }
            return array;
        }
        else {
            return messageDigest.digest(input);
        }
    }
    
    public int getMaxDataSize() {
        return this.maxDataSize;
    }
    
    public byte[] pad(byte[] array, int n, int n2) throws BadPaddingException {
        return this.pad(RSACore.convert(array, n, n2));
    }
    
    public byte[] pad(byte[] array) throws BadPaddingException {
        if (array.length > this.maxDataSize) {
            throw new BadPaddingException("Data must be shorter than " + (this.maxDataSize + 1) + " bytes but received " + array.length + " bytes.");
        }
        switch (this.type) {
            case 3: {
                return array;
            }
            case 1:
            case 2: {
                return this.padV15(array);
            }
            case 4: {
                return this.padOAEP(array);
            }
            default: {
                throw new AssertionError();
            }
        }
    }
    
    public byte[] unpad(byte[] array, int n, int n2) throws BadPaddingException {
        return this.unpad(RSACore.convert(array, n, n2));
    }
    
    public byte[] unpad(byte[] array) throws BadPaddingException {
        if (array.length != this.paddedSize) {
            throw new BadPaddingException("Decryption error.The padded array length (" + array.length + ") is not the specified padded size (" + this.paddedSize + ")");
        }
        switch (this.type) {
            case 3: {
                return array;
            }
            case 1:
            case 2: {
                return this.unpadV15(array);
            }
            case 4: {
                return this.unpadOAEP(array);
            }
            default: {
                throw new AssertionError();
            }
        }
    }
    
    private byte[] padV15(byte[] array) throws BadPaddingException {
        byte[] array2 = new byte[this.paddedSize];
        System.arraycopy(array, 0, array2, this.paddedSize - array.length, array.length);
        int n = this.paddedSize - 3 - array.length;
        int n2 = 0;
        array2[n2++] = 0;
        array2[n2++] = (byte)this.type;
        if (this.type == 1) {
            while (n-- > 0) {
                array2[n2++] = -1;
            }
            return array2;
        }
        if (this.random == null) {
            this.random = JCAUtil.getSecureRandom();
        }
        byte[] bytes = new byte[64];
        int n3 = -1;
        while (n-- > 0) {
            int i;
            do {
                if (n3 < 0) {
                    this.random.nextBytes(bytes);
                    n3 = bytes.length - 1;
                }
                i = (bytes[n3--] & 0xFF);
            } while (i == 0);
            array2[n2++] = (byte)i;
        }
        return array2;
    }
    
    private byte[] unpadV15(byte[] array) throws BadPaddingException {
        int i = 0;
        boolean n = false;
        if (array[i++] != 0) {
            n = true;
        }
        if (array[i++] != this.type) {
            n = true;
        }
        int n2 = 0;
        while (i < array.length) {
            int n3 = array[i++] & 0xFF;
            if (n3 == 0 && n2 == 0) {
                n2 = i;
            }
            if (i == array.length && n2 == 0) {
                n = true;
            }
            if (this.type == 1 && n3 != 255 && n2 == 0) {
                n = true;
            }
            else {
                continue;
            }
        }
        int n4 = array.length - n2;
        if (n4 > this.maxDataSize) {
            n = true;
        }
        System.arraycopy(array, 0, new byte[n2], 0, n2);
        byte[] array2 = new byte[n4];
        System.arraycopy(array, n2, array2, 0, n4);
        BadPaddingException o = new BadPaddingException("Decryption error");
        if (!(n == 0)) {
            throw o;
        }
        return array2;
    }
    
    private byte[] padOAEP(byte[] array) throws BadPaddingException {
        if (this.random == null) {
            this.random = JCAUtil.getSecureRandom();
        }
        int length = this.lHash.length;
        byte[] bytes = new byte[length];
        this.random.nextBytes(bytes);
        byte[] array2 = new byte[this.paddedSize];
        int n = 1;
        System.arraycopy(bytes, 0, array2, n, length);
        int n2 = length + 1;
        int n3 = array2.length - n2;
        int n4 = this.paddedSize - array.length;
        System.arraycopy(this.lHash, 0, array2, n2, length);
        array2[n4 - 1] = 1;
        System.arraycopy(array, 0, array2, n4, array.length);
        this.mgf.generateAndXor(array2, n, length, n3, array2, n2);
        this.mgf.generateAndXor(array2, n2, n3, length, array2, n);
        return array2;
    }
    
    private byte[] unpadOAEP(byte[] array) throws BadPaddingException {
        boolean n = false;
        int length = this.lHash.length;
        if (array[0] != 0) {
            n = true;
        }
        int n2 = 1;
        int n3 = length + 1;
        int n4 = array.length - n3;
        this.mgf.generateAndXor(array, n3, n4, length, array, n2);
        this.mgf.generateAndXor(array, n2, length, n4, array, n3);
        for (int i = 0; i < length; ++i) {
            if (this.lHash[i] != array[n3 + i]) {
                n = true;
            }
        }
        int j = n3 + length;
        int n5 = -1;
        for (; j < array.length; ++j) {
            int n6 = array[j];
            if (n5 == -1 && n6 != 0) {
                if (n6 == 1) {
                    n5 = j;
                }
                else {
                    n = true;
                }
            }
        }
        if (n5 == -1) {
            n = true;
            n5 = array.length - 1;
        }
        int n7 = n5 + 1;
        byte[] array2 = new byte[n7 - j];
        System.arraycopy(array, j, array2, 0, array2.length);
        byte[] array3 = new byte[array.length - n7];
        System.arraycopy(array, n7, array3, 0, array3.length);
        BadPaddingException o = new BadPaddingException("Decryption error");
        if (!(n == 0)) {
            throw o;
        }
        return array3;
    }
    
    static {
        emptyHashes = Collections.synchronizedMap(new HashMap());
    }
}
